'  MIT License:   See end of file for terms of use.

'Notes:    Original code by Rayman   adapted for keypad project


CON
  _CLKMODE      = XTAL1 + PLL16X                       
  _XINFREQ      = 5_000_000

CON 'SD card pin settings
  SD_MISO=27
  SD_Clk=26
  SD_MOSI=25
  SD_CS=24
  

  

  

OBJ
  lcd : "QMP_DisplayDriver2d"  'Controls the LCD
  sd: "fsrw"     'SD card driver
  ser : "fullduplexserial" 'for monitoring
  i2c: "QMP_I2CDriver"  


VAR  'Display variables
  long foregroundcolor,backgroundcolor
   byte x
VAR 'buffer for bitmap line of pixels
  long lineBuffer[320*3/4]' buffer for line of an image and scrolling  'needs to be long aligned
  long BmpPalette[256]
  


VAR 'touchscreen
  long bStillDown, touchx, touchy
  
DAT 'define on-screen buttons
nButtons byte 2
Buttons byte
buttonPlay byte 238,290,49,82,1  'x1,x2,y1,y2,handler 1=playbutton result in Getbutton routine
buttonStop byte 65,95,179,209,2  'coords of stop button  2=stopbutton

DAT 'Default Touchscreen Calibrations
'based on these reading:
'  Top of landscape screen: x=60600, Bottom of landscape screen: x=4560   
'  Left of landscape screen (screen only, not buttons): y=6200,  Right of landscape screen:  y=60600
'  Left of button area:  (y=1500)  
touchx0 long 4560
touchdx long (60600-4560)/240
touchy0 long 6200
touchdy long (60600-6200)/320
    
pub Main|i,j               'Startup Routine
  'Start serial connection
  ser.start(31,30,0,115200)
  'initialize the LCD
  lcd.start
  'init touch
  i2c.StartTouch
  
  'set default colors
  backgroundcolor:=lcd.RGB(255,0,0) 'Red
  foregroundcolor:=lcd.RGB(255,255,0) 'yellow
  'Fill screen fast way
  lcd.FillScreen(backgroundcolor)
  lcd.SetFontColors(foregroundcolor,backgroundcolor)   
  'Change font
  lcd.SetFont(@Font1)  

  

  'Mounting SD card now
  
  repeat   'try forever to mount the SD card
    i:=\sd.Mount_Explicit(SD_MISO,SD_CLK,SD_MOSI,SD_CS) 
    if i==0
       quit  'mounted, so exit
    waitcnt(cnt+clkfreq/4)
    ser.tx(".")
  bmp(0,0,string("FrLitD.bmp"))
  'bmp(50,240-51-20,string("PBGrey.bmp"))  'Play button with light grey background
  'lcd.DrawString(110,17,string("QMP Wav Player"))
  
  lcd.DrawRectangle(lcd#rgb_red,buttonPlay[0],buttonPlay[2],buttonPlay[1],buttonPlay[3])  'see where button is
  'lcd.DrawRectangle(lcd#rgb_red,buttonStop[0],buttonStop[2],buttonStop[1],buttonStop[3])
  
     calc
PuB calc 

  repeat 
      
    WaitStylusDown

     
    
    ser.tx(0)
    ser.tx(1)
    ser.dec(x)
    ser.tx(" ")
    ser.dec(touchx)
    ser.tx(",")
    ser.dec(touchy)
   
    
   if GetButton==1  and x<255
     x++
     
       
 
     
   if GetButton==2 and x>0
     x--
     
      
     
     
     
   waitcnt(cnt+clkfreq/25)
  
  

PUB GetButton|i,p 'see if a button was pushed
  p:=@Buttons
  repeat i from 0 to nButtons-1
    if touchx>byte[p] and touchx<byte[p+1]
      if touchy>byte[p+2] and touchy<byte[p+3]
        return byte[p+4]
    p+=5
  
  return 0  'no button found
  

PUB bTouchDown|tx,ty,z1,z2,tz  
  tx:=i2c.GetTouchX
  z1:=i2c.GetTouchZ1
  z2:=i2c.GetTouchZ2
  tz:=tx*(z2-z1)/z1
  if (tz<90000) and (tz>10000)           'You might have to tweak these values for some displays...
    return true
  else
    return false

PUB WaitStylusUp
  'Wait for stylus to be up
  waitcnt(cnt+clkfreq/10)
  repeat
   if !bTouchDown
    return
  
 

    
PUB WaitStylusDown|tx,ty,z1,z2,tz      'Note: x&y reversed in landscape mode 
  'Wait for stylus to be down then update touchx, touchy and return
  repeat
    if bTouchDown
      tx:=i2c.GetTouchX
      ty:=i2c.GetTouchY
      z1:=i2c.GetTouchZ1
      z2:=i2c.GetTouchZ2
      'take another x,y reading and average
      tx+=i2c.GetTouchX
      ty+=i2c.GetTouchY
      tx>>=1
      ty>>=1
      tz:=tx*(z2-z1)/z1
      if (tz<90000) and (tz>10000)
        touchy:=240-(tx-touchx0)/touchdx      'Note: x&y reversed in landscape mode
        touchx:=(ty-touchy0)/touchdy
        return
      else
        bStillDown:=false
    else
      bStillDown:=false   



PUB BMP(bmpX,bmpY,sFilename)|i,j,k,l
'Draw a bitmap file onto the screen
    'Update speed seems to be only limited by SD card reading speed
    'note that bitmaps are stored from bottom line to top line!
    i:=\sd.popen(sFilename,"r") 'open image for reading
    if i<>0
      return i   
    sd.pread(@bfType,2) 'read bmp header
    sd.pread(@bfSize,4) 'read bmp header  
    sd.pread(@bfReserved1,4) 'read bmp header  
    sd.pread(@bfOffBits,16) 'read bmp header
    sd.pread(@biPlanes,4)
    sd.pread(@biCompression,24)

    

    if (biHeight<1) or (biWidth<1)
      return -1  'some kind of problem

    l:=biHeight+bmpY-1  
      
    case biBitCount
      24:  'true color (3 bytes per pixel)
        i:=(biWidth*3)//4
        k:=biWidth*3+i
        repeat j from l to bmpY 
          'process image row by row          
          sd.pRead(@lineBuffer,k)

          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,24,0)
 
      8:  '256-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54
        'read in palette
        sd.pread(@BmpPalette,i)
        i:=biWidth//4
        k:=biWidth+i
        repeat j from l to bmpY          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,8,@BmpPalette) 
      
      4:  '16-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54        
        'read in palette
        sd.pread(@BmpPalette,i)
        'now paint picture
        i:=(biWidth>>1)//4
        k:=(biWidth>>1)+i 
        repeat j from l to bmpY          
          sd.pRead(@lineBuffer,k)
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1)) 
          lcd.WriteBitmapLine(@lineBuffer,biWidth,4,@BmpPalette) 

    sd.pclose
    return 0
 




DAT     'Data for reading bitmap files

BMPHeader  byte 'Mostly using info from here:  http://www.fortunecity.com/skyscraper/windows/364/bmpffrmt.html
bfType byte "B","M"  ' 19778
bfSize long 0
bfReserved1 word 0
bfReserved2 word 0
bfOffBits long 54
biSize long 40
biWidth long 0
biHeight long 10
biPlanes word 1
biBitCount word 24
biCompression long 0
biSizeImage long 0
biXPelsPerMeter long 0
biYPelsPerMeter long 0
biClrUsed long 0
biClrImportant long 0    


DAT  'Font:  Custom font located here.  Other fonts can be selected with the SetFont command
'Note:  If you do not need a font, you can save space by commenting out the file command line below
        
font1 long
file "ProgoCo_wHeader.dat"   'nice 8x16 fixed pitch font

DAT   'for storing filename from SD card
fname   byte "i"[13],0



DAT 'MIT License
{{
'  Permission is hereby granted, free of charge, to any person obtaining
'  a copy of this software and associated documentation files
'  (the "Software"), to deal in the Software without restriction,
'  including without limitation the rights to use, copy, modify, merge,
'  publish, distribute, sublicense, and/or sell copies of the Software,
'  and to permit persons to whom the Software is furnished to do so,
'  subject to the following conditions:
'
'  The above copyright notice and this permission notice shall be included
'  in all copies or substantial portions of the Software.
'
'  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
'  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
'  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
'  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
'  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
'  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
'  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}}